@@ -9,20 +9,20 @@ fn dump_err(py: Python, e: PyErr) {
 }
 
 fn main() {
-    pyo3::prepare_freethreaded_python();
+    Python::initialize();
 
-    Python::with_gil(|py| {
-        let asyncio = py.import_bound("asyncio")?;
+    Python::attach(|py| {
+        let asyncio = py.import("asyncio")?;
 
         let event_loop = asyncio.call_method0("new_event_loop")?;
         asyncio.call_method1("set_event_loop", (&event_loop,))?;
 
-        let event_loop_hdl = PyObject::from(event_loop.clone());
+        let event_loop_hdl: Py<PyAny> = event_loop.clone().into();
 
         async_std::task::spawn(async move {
             async_std::task::sleep(Duration::from_secs(1)).await;
 
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 event_loop_hdl
                     .bind(py)
                     .call_method1(
@@ -43,6 +43,6 @@ fn main() {
         println!("test test_async_std_run_forever ... ok");
         Ok(())
     })
-    .map_err(|e| Python::with_gil(|py| dump_err(py, e)))
+    .map_err(|e| Python::attach(|py| dump_err(py, e)))
     .unwrap()
 }
