@@ -9,7 +9,7 @@
 //!
 //! ```toml
 //! [dependencies.pyo3-async-runtimes]
-//! version = "0.22"
+//! version = "0.24"
 //! features = ["unstable-streams"]
 //! ```
 
@@ -27,10 +27,9 @@ use crate::{
 use futures::channel::oneshot;
 #[cfg(feature = "unstable-streams")]
 use futures::{channel::mpsc, SinkExt};
-#[cfg(feature = "unstable-streams")]
-use once_cell::sync::OnceCell;
 use pin_project_lite::pin_project;
 use pyo3::prelude::*;
+use pyo3::IntoPyObjectExt;
 #[cfg(feature = "unstable-streams")]
 use std::marker::PhantomData;
 
@@ -180,8 +179,8 @@ where
 /// #
 /// # use pyo3::prelude::*;
 /// #
-/// # Python::with_gil(|py| -> PyResult<()> {
-/// # let event_loop = py.import_bound("asyncio")?.call_method0("new_event_loop")?;
+/// # Python::attach(|py| -> PyResult<()> {
+/// # let event_loop = py.import("asyncio")?.call_method0("new_event_loop")?;
 /// # #[cfg(feature = "tokio-runtime")]
 /// pyo3_async_runtimes::generic::run_until_complete::<MyCustomRuntime, _, _>(&event_loop, async move {
 ///     tokio::time::sleep(Duration::from_secs(1)).await;
@@ -286,7 +285,7 @@ where
 /// # use pyo3::prelude::*;
 /// #
 /// fn main() {
-///     Python::with_gil(|py| {
+///     Python::attach(|py| {
 ///         pyo3_async_runtimes::generic::run::<MyCustomRuntime, _, _>(py, async move {
 ///             custom_sleep(Duration::from_secs(1)).await;
 ///             Ok(())
@@ -341,14 +340,14 @@ impl CheckedCompletor {
 fn set_result(
     event_loop: &Bound<PyAny>,
     future: &Bound<PyAny>,
-    result: PyResult<PyObject>,
+    result: PyResult<Py<PyAny>>,
 ) -> PyResult<()> {
     let py = event_loop.py();
     let none = py.None().into_bound(py);
 
     let (complete, val) = match result {
-        Ok(val) => (future.getattr("set_result")?, val.into_py(py)),
-        Err(err) => (future.getattr("set_exception")?, err.into_py(py)),
+        Ok(val) => (future.getattr("set_result")?, val.into_pyobject(py)?),
+        Err(err) => (future.getattr("set_exception")?, err.into_bound_py_any(py)?),
     };
     call_soon_threadsafe(event_loop, &none, (CheckedCompletor, future, complete, val))?;
 
@@ -368,6 +367,7 @@ fn set_result(
 ///
 /// ```no_run
 /// # use std::{any::Any, pin::Pin, future::Future, task::{Context, Poll}, time::Duration};
+/// # use std::ffi::CString;
 /// #
 /// # use pyo3::prelude::*;
 /// #
@@ -437,22 +437,22 @@ fn set_result(
 /// "#;
 ///
 /// async fn py_sleep(seconds: f32) -> PyResult<()> {
-///     let test_mod = Python::with_gil(|py| -> PyResult<PyObject> {
+///     let test_mod = Python::attach(|py| -> PyResult<Py<PyAny>> {
 ///         Ok(
-///             PyModule::from_code_bound(
+///             PyModule::from_code(
 ///                 py,
-///                 PYTHON_CODE,
-///                 "test_into_future/test_mod.py",
-///                 "test_mod"
+///                 &CString::new(PYTHON_CODE).unwrap(),
+///                 &CString::new("test_into_future/test_mod.py").unwrap(),
+///                 &CString::new("test_mod").unwrap(),
 ///             )?
 ///             .into()
 ///         )
 ///     })?;
 ///
-///     Python::with_gil(|py| {
+///     Python::attach(|py| {
 ///         pyo3_async_runtimes::generic::into_future::<MyCustomRuntime>(
 ///             test_mod
-///                 .call_method1(py, "py_sleep", (seconds.into_py(py),))?
+///                 .call_method1(py, "py_sleep", (seconds,))?
 ///                 .into_bound(py),
 ///         )
 ///     })?
@@ -462,7 +462,7 @@ fn set_result(
 /// ```
 pub fn into_future<R>(
     awaitable: Bound<PyAny>,
-) -> PyResult<impl Future<Output = PyResult<PyObject>> + Send>
+) -> PyResult<impl Future<Output = PyResult<Py<PyAny>>> + Send>
 where
     R: Runtime + ContextExt,
 {
@@ -581,7 +581,7 @@ pub fn future_into_py_with_locals<R, F, T>(
 where
     R: Runtime + ContextExt,
     F: Future<Output = PyResult<T>> + Send + 'static,
-    T: IntoPy<PyObject>,
+    T: for<'py> IntoPyObject<'py>,
 {
     let (cancel_tx, cancel_rx) = oneshot::channel();
 
@@ -593,20 +593,20 @@ where
         },),
     )?;
 
-    let future_tx1 = PyObject::from(py_fut.clone());
+    let future_tx1: Py<PyAny> = py_fut.clone().into();
     let future_tx2 = future_tx1.clone_ref(py);
 
     R::spawn(async move {
-        let locals2 = Python::with_gil(|py| locals.clone_ref(py));
+        let locals2 = Python::attach(|py| locals.clone_ref(py));
 
         if let Err(e) = R::spawn(async move {
             let result = R::scope(
-                Python::with_gil(|py| locals2.clone_ref(py)),
+                Python::attach(|py| locals2.clone_ref(py)),
                 Cancellable::new_with_cancel_rx(fut, cancel_rx),
             )
             .await;
 
-            Python::with_gil(move |py| {
+            Python::attach(move |py| {
                 if cancelled(future_tx1.bind(py))
                     .map_err(dump_err(py))
                     .unwrap_or(false)
@@ -617,7 +617,7 @@ where
                 let _ = set_result(
                     &locals2.event_loop(py),
                     future_tx1.bind(py),
-                    result.map(|val| val.into_py(py)),
+                    result.and_then(|val| val.into_py_any(py)),
                 )
                 .map_err(dump_err(py));
             });
@@ -625,7 +625,7 @@ where
         .await
         {
             if e.is_panic() {
-                Python::with_gil(move |py| {
+                Python::attach(move |py| {
                     if cancelled(future_tx2.bind(py))
                         .map_err(dump_err(py))
                         .unwrap_or(false)
@@ -686,10 +686,10 @@ impl<T> Cancellable<T> {
     }
 }
 
-impl<F, T> Future for Cancellable<F>
+impl<'py, F, T> Future for Cancellable<F>
 where
     F: Future<Output = PyResult<T>>,
-    T: IntoPy<PyObject>,
+    T: IntoPyObject<'py>,
 {
     type Output = F::Output;
 
@@ -844,7 +844,7 @@ pub fn future_into_py<R, F, T>(py: Python, fut: F) -> PyResult<Bound<PyAny>>
 where
     R: Runtime + ContextExt,
     F: Future<Output = PyResult<T>> + Send + 'static,
-    T: IntoPy<PyObject>,
+    T: for<'py> IntoPyObject<'py>,
 {
     future_into_py_with_locals::<R, F, T>(py, get_current_locals::<R>(py)?, fut)
 }
@@ -986,7 +986,7 @@ pub fn local_future_into_py_with_locals<R, F, T>(
 where
     R: Runtime + SpawnLocalExt + LocalContextExt,
     F: Future<Output = PyResult<T>> + 'static,
-    T: IntoPy<PyObject>,
+    T: for<'py> IntoPyObject<'py>,
 {
     let (cancel_tx, cancel_rx) = oneshot::channel();
 
@@ -998,20 +998,20 @@ where
         },),
     )?;
 
-    let future_tx1 = PyObject::from(py_fut.clone());
+    let future_tx1: Py<PyAny> = py_fut.clone().into();
     let future_tx2 = future_tx1.clone_ref(py);
 
     R::spawn_local(async move {
-        let locals2 = Python::with_gil(|py| locals.clone_ref(py));
+        let locals2 = Python::attach(|py| locals.clone_ref(py));
 
         if let Err(e) = R::spawn_local(async move {
             let result = R::scope_local(
-                Python::with_gil(|py| locals2.clone_ref(py)),
+                Python::attach(|py| locals2.clone_ref(py)),
                 Cancellable::new_with_cancel_rx(fut, cancel_rx),
             )
             .await;
 
-            Python::with_gil(move |py| {
+            Python::attach(move |py| {
                 if cancelled(future_tx1.bind(py))
                     .map_err(dump_err(py))
                     .unwrap_or(false)
@@ -1022,7 +1022,7 @@ where
                 let _ = set_result(
                     locals2.event_loop.bind(py),
                     future_tx1.bind(py),
-                    result.map(|val| val.into_py(py)),
+                    result.and_then(|val| val.into_py_any(py)),
                 )
                 .map_err(dump_err(py));
             });
@@ -1030,7 +1030,7 @@ where
         .await
         {
             if e.is_panic() {
-                Python::with_gil(move |py| {
+                Python::attach(move |py| {
                     if cancelled(future_tx2.bind(py))
                         .map_err(dump_err(py))
                         .unwrap_or(false)
@@ -1183,7 +1183,7 @@ pub fn local_future_into_py<R, F, T>(py: Python, fut: F) -> PyResult<Bound<PyAny
 where
     R: Runtime + ContextExt + SpawnLocalExt + LocalContextExt,
     F: Future<Output = PyResult<T>> + 'static,
-    T: IntoPy<PyObject>,
+    T: for<'py> IntoPyObject<'py>,
 {
     local_future_into_py_with_locals::<R, F, T>(py, get_current_locals::<R>(py)?, fut)
 }
@@ -1256,6 +1256,7 @@ where
 ///
 /// use pyo3::prelude::*;
 /// use futures::{StreamExt, TryStreamExt};
+/// use std::ffi::CString;
 ///
 /// const TEST_MOD: &str = r#"
 /// import asyncio
@@ -1267,12 +1268,12 @@ where
 /// "#;
 ///
 /// # async fn test_async_gen() -> PyResult<()> {
-/// let stream = Python::with_gil(|py| {
-///     let test_mod = PyModule::from_code_bound(
+/// let stream = Python::attach(|py| {
+///     let test_mod = PyModule::from_code(
 ///         py,
-///         TEST_MOD,
-///         "test_rust_coroutine/test_mod.py",
-///         "test_mod",
+///         &CString::new(TEST_MOD).unwrap(),
+///         &CString::new("test_rust_coroutine/test_mod.py").unwrap(),
+///         &CString::new("test_mod").unwrap(),
 ///     )?;
 ///
 ///     pyo3_async_runtimes::generic::into_stream_with_locals_v1::<MyCustomRuntime>(
@@ -1282,7 +1283,7 @@ where
 /// })?;
 ///
 /// let vals = stream
-///     .map(|item| Python::with_gil(|py| -> PyResult<i32> { Ok(item?.bind(py).extract()?) }))
+///     .map(|item| Python::attach(|py| -> PyResult<i32> { Ok(item?.bind(py).extract()?) }))
 ///     .try_collect::<Vec<i32>>()
 ///     .await?;
 ///
@@ -1296,23 +1297,23 @@ where
 pub fn into_stream_with_locals_v1<R>(
     locals: TaskLocals,
     gen: Bound<'_, PyAny>,
-) -> PyResult<impl futures::Stream<Item = PyResult<PyObject>> + 'static>
+) -> PyResult<impl futures::Stream<Item = PyResult<Py<PyAny>>> + 'static>
 where
     R: Runtime,
 {
     let (tx, rx) = async_channel::bounded(1);
-    let anext = PyObject::from(gen.getattr("__anext__")?);
+    let anext: Py<PyAny> = gen.getattr("__anext__")?.into();
 
     R::spawn(async move {
         loop {
-            let fut = Python::with_gil(|py| -> PyResult<_> {
+            let fut = Python::attach(|py| -> PyResult<_> {
                 into_future_with_locals(&locals, anext.bind(py).call0()?)
             });
             let item = match fut {
                 Ok(fut) => match fut.await {
                     Ok(item) => Ok(item),
                     Err(e) => {
-                        let stop_iter = Python::with_gil(|py| {
+                        let stop_iter = Python::attach(|py| {
                             e.is_instance_of::<pyo3::exceptions::PyStopAsyncIteration>(py)
                         });
 
@@ -1404,6 +1405,7 @@ where
 ///
 /// use pyo3::prelude::*;
 /// use futures::{StreamExt, TryStreamExt};
+/// use std::ffi::CString;
 ///
 /// const TEST_MOD: &str = r#"
 /// import asyncio
@@ -1415,19 +1417,19 @@ where
 /// "#;
 ///
 /// # async fn test_async_gen() -> PyResult<()> {
-/// let stream = Python::with_gil(|py| {
-///     let test_mod = PyModule::from_code_bound(
+/// let stream = Python::attach(|py| {
+///     let test_mod = PyModule::from_code(
 ///         py,
-///         TEST_MOD,
-///         "test_rust_coroutine/test_mod.py",
-///         "test_mod",
+///         &CString::new(TEST_MOD).unwrap(),
+///         &CString::new("test_rust_coroutine/test_mod.py").unwrap(),
+///         &CString::new("test_mod").unwrap(),
 ///     )?;
 ///
 ///     pyo3_async_runtimes::generic::into_stream_v1::<MyCustomRuntime>(test_mod.call_method0("gen")?)
 /// })?;
 ///
 /// let vals = stream
-///     .map(|item| Python::with_gil(|py| -> PyResult<i32> { Ok(item?.bind(py).extract()?) }))
+///     .map(|item| Python::attach(|py| -> PyResult<i32> { Ok(item?.bind(py).extract()?) }))
 ///     .try_collect::<Vec<i32>>()
 ///     .await?;
 ///
@@ -1439,7 +1441,7 @@ where
 #[cfg(feature = "unstable-streams")]
 pub fn into_stream_v1<R>(
     gen: Bound<'_, PyAny>,
-) -> PyResult<impl futures::Stream<Item = PyResult<PyObject>> + 'static>
+) -> PyResult<impl futures::Stream<Item = PyResult<Py<PyAny>>> + 'static>
 where
     R: Runtime + ContextExt,
 {
@@ -1447,7 +1449,7 @@ where
 }
 
 trait Sender: Send + 'static {
-    fn send(&mut self, py: Python, locals: TaskLocals, item: PyObject) -> PyResult<PyObject>;
+    fn send(&mut self, py: Python, locals: TaskLocals, item: Py<PyAny>) -> PyResult<Py<PyAny>>;
     fn close(&mut self) -> PyResult<()>;
 }
 
@@ -1457,7 +1459,7 @@ where
     R: Runtime,
 {
     runtime: PhantomData<R>,
-    tx: mpsc::Sender<PyObject>,
+    tx: mpsc::Sender<Py<PyAny>>,
 }
 
 #[cfg(feature = "unstable-streams")]
@@ -1465,30 +1467,27 @@ impl<R> Sender for GenericSender<R>
 where
     R: Runtime + ContextExt,
 {
-    fn send(&mut self, py: Python, locals: TaskLocals, item: PyObject) -> PyResult<PyObject> {
+    fn send(&mut self, py: Python, locals: TaskLocals, item: Py<PyAny>) -> PyResult<Py<PyAny>> {
         match self.tx.try_send(item.clone_ref(py)) {
-            Ok(_) => Ok(true.into_py(py)),
+            Ok(_) => true.into_py_any(py),
             Err(e) => {
                 if e.is_full() {
                     let mut tx = self.tx.clone();
-                    Python::with_gil(move |py| {
-                        Ok(
-                            future_into_py_with_locals::<R, _, PyObject>(py, locals, async move {
-                                if tx.flush().await.is_err() {
-                                    // receiving side disconnected
-                                    return Python::with_gil(|py| Ok(false.into_py(py)));
-                                }
-                                if tx.send(item).await.is_err() {
-                                    // receiving side disconnected
-                                    return Python::with_gil(|py| Ok(false.into_py(py)));
-                                }
-                                Python::with_gil(|py| Ok(true.into_py(py)))
-                            })?
-                            .into(),
-                        )
+
+                    future_into_py_with_locals::<R, _, bool>(py, locals, async move {
+                        if tx.flush().await.is_err() {
+                            // receiving side disconnected
+                            return Ok(false);
+                        }
+                        if tx.send(item).await.is_err() {
+                            // receiving side disconnected
+                            return Ok(false);
+                        }
+                        Ok(true)
                     })
+                    .map(Bound::unbind)
                 } else {
-                    Ok(false.into_py(py))
+                    false.into_py_any(py)
                 }
             }
         }
@@ -1502,15 +1501,20 @@ where
 #[pyclass]
 struct SenderGlue {
     locals: TaskLocals,
-    tx: Box<dyn Sender>,
+    tx: Arc<Mutex<dyn Sender>>,
 }
 #[pymethods]
 impl SenderGlue {
-    pub fn send(&mut self, item: PyObject) -> PyResult<PyObject> {
-        Python::with_gil(|py| self.tx.send(py, self.locals.clone_ref(py), item))
+    pub fn send(&mut self, item: Py<PyAny>) -> PyResult<Py<PyAny>> {
+        Python::attach(|py| {
+            self.tx
+                .lock()
+                .unwrap()
+                .send(py, self.locals.clone_ref(py), item)
+        })
     }
     pub fn close(&mut self) -> PyResult<()> {
-        self.tx.close()
+        self.tx.lock().unwrap().close()
     }
 }
 
@@ -1601,6 +1605,7 @@ async def forward(gen, sender):
 ///
 /// use pyo3::prelude::*;
 /// use futures::{StreamExt, TryStreamExt};
+/// use std::ffi::CString;
 ///
 /// const TEST_MOD: &str = r#"
 /// import asyncio
@@ -1612,12 +1617,12 @@ async def forward(gen, sender):
 /// "#;
 ///
 /// # async fn test_async_gen() -> PyResult<()> {
-/// let stream = Python::with_gil(|py| {
-///     let test_mod = PyModule::from_code_bound(
+/// let stream = Python::attach(|py| {
+///     let test_mod = PyModule::from_code(
 ///         py,
-///         TEST_MOD,
-///         "test_rust_coroutine/test_mod.py",
-///         "test_mod",
+///         &CString::new(TEST_MOD).unwrap(),
+///         &CString::new("test_rust_coroutine/test_mod.py").unwrap(),
+///         &CString::new("test_mod").unwrap(),
 ///     )?;
 ///
 ///     pyo3_async_runtimes::generic::into_stream_with_locals_v2::<MyCustomRuntime>(
@@ -1627,7 +1632,7 @@ async def forward(gen, sender):
 /// })?;
 ///
 /// let vals = stream
-///     .map(|item| Python::with_gil(|py| -> PyResult<i32> { Ok(item.bind(py).extract()?) }))
+///     .map(|item| Python::attach(|py| -> PyResult<i32> { Ok(item.bind(py).extract()?) }))
 ///     .try_collect::<Vec<i32>>()
 ///     .await?;
 ///
@@ -1640,19 +1645,23 @@ async def forward(gen, sender):
 pub fn into_stream_with_locals_v2<R>(
     locals: TaskLocals,
     gen: Bound<'_, PyAny>,
-) -> PyResult<impl futures::Stream<Item = PyObject> + 'static>
+) -> PyResult<impl futures::Stream<Item = Py<PyAny>> + 'static>
 where
     R: Runtime + ContextExt,
 {
-    static GLUE_MOD: OnceCell<PyObject> = OnceCell::new();
+    use std::ffi::CString;
+
+    use pyo3::sync::PyOnceLock;
+
+    static GLUE_MOD: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     let py = gen.py();
     let glue = GLUE_MOD
-        .get_or_try_init(|| -> PyResult<PyObject> {
-            Ok(PyModule::from_code_bound(
+        .get_or_try_init(py, || -> PyResult<Py<PyAny>> {
+            Ok(PyModule::from_code(
                 py,
-                STREAM_GLUE,
-                "pyo3_async_runtimes/pyo3_async_runtimes_glue.py",
-                "pyo3_async_runtimes_glue",
+                &CString::new(STREAM_GLUE).unwrap(),
+                &CString::new("pyo3_async_runtimes/pyo3_async_runtimes_glue.py").unwrap(),
+                &CString::new("pyo3_async_runtimes_glue").unwrap(),
             )?
             .into())
         })?
@@ -1670,10 +1679,10 @@ where
                     gen,
                     SenderGlue {
                         locals,
-                        tx: Box::new(GenericSender {
+                        tx: Arc::new(Mutex::new(GenericSender {
                             runtime: PhantomData::<R>,
                             tx,
-                        }),
+                        })),
                     },
                 ),
             )?,
@@ -1749,6 +1758,7 @@ where
 ///
 /// use pyo3::prelude::*;
 /// use futures::{StreamExt, TryStreamExt};
+/// use std::ffi::CString;
 ///
 /// const TEST_MOD: &str = r#"
 /// import asyncio
@@ -1760,19 +1770,19 @@ where
 /// "#;
 ///
 /// # async fn test_async_gen() -> PyResult<()> {
-/// let stream = Python::with_gil(|py| {
-///     let test_mod = PyModule::from_code_bound(
+/// let stream = Python::attach(|py| {
+///     let test_mod = PyModule::from_code(
 ///         py,
-///         TEST_MOD,
-///         "test_rust_coroutine/test_mod.py",
-///         "test_mod",
+///         &CString::new(TEST_MOD).unwrap(),
+///         &CString::new("test_rust_coroutine/test_mod.py").unwrap(),
+///         &CString::new("test_mod").unwrap(),
 ///     )?;
 ///
 ///     pyo3_async_runtimes::generic::into_stream_v2::<MyCustomRuntime>(test_mod.call_method0("gen")?)
 /// })?;
 ///
 /// let vals = stream
-///     .map(|item| Python::with_gil(|py| -> PyResult<i32> { Ok(item.bind(py).extract()?) }))
+///     .map(|item| Python::attach(|py| -> PyResult<i32> { Ok(item.bind(py).extract()?) }))
 ///     .try_collect::<Vec<i32>>()
 ///     .await?;
 ///
@@ -1784,7 +1794,7 @@ where
 #[cfg(feature = "unstable-streams")]
 pub fn into_stream_v2<R>(
     gen: Bound<'_, PyAny>,
-) -> PyResult<impl futures::Stream<Item = PyObject> + 'static>
+) -> PyResult<impl futures::Stream<Item = Py<PyAny>> + 'static>
 where
     R: Runtime + ContextExt,
 {
