@@ -94,13 +94,13 @@
 //!
 //!     // Convert the async move { } block to a Python awaitable
 //!     pyo3_async_runtimes::tokio::future_into_py_with_locals(py, locals.clone_ref(py), async move {
-//!         let py_sleep = Python::with_gil(|py| {
+//!         let py_sleep = Python::attach(|py| {
 //!             // Sometimes we need to call other async Python functions within
 //!             // this future. In order for this to work, we need to track the
 //!             // event loop from earlier.
 //!             pyo3_async_runtimes::into_future_with_locals(
 //!                 &locals,
-//!                 py.import_bound("asyncio")?.call_method1("sleep", (1,))?
+//!                 py.import("asyncio")?.call_method1("sleep", (1,))?
 //!             )
 //!         })?;
 //!
@@ -165,17 +165,17 @@
 //!         locals.clone_ref(py),
 //!         // Store the current locals in task-local data
 //!         pyo3_async_runtimes::tokio::scope(locals.clone_ref(py), async move {
-//!             let py_sleep = Python::with_gil(|py| {
+//!             let py_sleep = Python::attach(|py| {
 //!                 pyo3_async_runtimes::into_future_with_locals(
 //!                     // Now we can get the current locals through task-local data
 //!                     &pyo3_async_runtimes::tokio::get_current_locals(py)?,
-//!                     py.import_bound("asyncio")?.call_method1("sleep", (1,))?
+//!                     py.import("asyncio")?.call_method1("sleep", (1,))?
 //!                 )
 //!             })?;
 //!
 //!             py_sleep.await?;
 //!
-//!             Ok(Python::with_gil(|py| py.None()))
+//!             Ok(Python::attach(|py| py.None()))
 //!         })
 //!     )
 //! }
@@ -192,7 +192,7 @@
 //!         locals.clone_ref(py),
 //!         // Store the current locals in task-local data
 //!         pyo3_async_runtimes::tokio::scope(locals.clone_ref(py), async move {
-//!             let py_sleep = Python::with_gil(|py| {
+//!             let py_sleep = Python::attach(|py| {
 //!                 pyo3_async_runtimes::into_future_with_locals(
 //!                     &pyo3_async_runtimes::tokio::get_current_locals(py)?,
 //!                     // We can also call sleep within a Rust task since the
@@ -203,7 +203,7 @@
 //!
 //!             py_sleep.await?;
 //!
-//!             Ok(Python::with_gil(|py| py.None()))
+//!             Ok(Python::attach(|py| py.None()))
 //!         })
 //!     )
 //! }
@@ -242,15 +242,15 @@
 //! #[pyfunction]
 //! fn sleep(py: Python) -> PyResult<Bound<PyAny>> {
 //!     pyo3_async_runtimes::tokio::future_into_py(py, async move {
-//!         let py_sleep = Python::with_gil(|py| {
+//!         let py_sleep = Python::attach(|py| {
 //!             pyo3_async_runtimes::tokio::into_future(
-//!                 py.import_bound("asyncio")?.call_method1("sleep", (1,))?
+//!                 py.import("asyncio")?.call_method1("sleep", (1,))?
 //!             )
 //!         })?;
 //!
 //!         py_sleep.await?;
 //!
-//!         Ok(Python::with_gil(|py| py.None()))
+//!         Ok(Python::attach(|py| py.None()))
 //!     })
 //! }
 //!
@@ -258,13 +258,13 @@
 //! #[pyfunction]
 //! fn wrap_sleep(py: Python) -> PyResult<Bound<PyAny>> {
 //!     pyo3_async_runtimes::tokio::future_into_py(py, async move {
-//!         let py_sleep = Python::with_gil(|py| {
+//!         let py_sleep = Python::attach(|py| {
 //!             pyo3_async_runtimes::tokio::into_future(sleep(py)?)
 //!         })?;
 //!
 //!         py_sleep.await?;
 //!
-//!         Ok(Python::with_gil(|py| py.None()))
+//!         Ok(Python::attach(|py| py.None()))
 //!     })
 //! }
 //!
@@ -300,7 +300,7 @@
 //!
 //! ```toml
 //! [dependencies.pyo3-async-runtimes]
-//! version = "0.22"
+//! version = "0.24"
 //! features = ["attributes"]
 //! ```
 //!
@@ -313,7 +313,7 @@
 //!
 //! ```toml
 //! [dependencies.pyo3-async-runtimes]
-//! version = "0.22"
+//! version = "0.24"
 //! features = ["async-std-runtime"]
 //! ```
 //!
@@ -326,7 +326,7 @@
 //!
 //! ```toml
 //! [dependencies.pyo3-async-runtimes]
-//! version = "0.22"
+//! version = "0.24"
 //! features = ["tokio-runtime"]
 //! ```
 //!
@@ -339,7 +339,7 @@
 //!
 //! ```toml
 //! [dependencies.pyo3-async-runtimes]
-//! version = "0.22"
+//! version = "0.24"
 //! features = ["testing"]
 //! ```
 
@@ -364,7 +364,7 @@ pub mod generic;
 
 #[pymodule]
 fn pyo3_async_runtimes(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
-    m.add("RustPanic", py.get_type_bound::<err::RustPanic>())?;
+    m.add("RustPanic", py.get_type::<err::RustPanic>())?;
     Ok(())
 }
 
@@ -397,27 +397,23 @@ pub mod doc_test {
 use std::future::Future;
 
 use futures::channel::oneshot;
-use once_cell::sync::OnceCell;
-use pyo3::{
-    prelude::*,
-    types::{PyDict, PyTuple},
-};
+use pyo3::{call::PyCallArgs, prelude::*, sync::PyOnceLock, types::PyDict};
 
-static ASYNCIO: OnceCell<PyObject> = OnceCell::new();
-static CONTEXTVARS: OnceCell<PyObject> = OnceCell::new();
-static ENSURE_FUTURE: OnceCell<PyObject> = OnceCell::new();
-static GET_RUNNING_LOOP: OnceCell<PyObject> = OnceCell::new();
+static ASYNCIO: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+static CONTEXTVARS: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+static ENSURE_FUTURE: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+static GET_RUNNING_LOOP: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
 
 fn ensure_future<'p>(py: Python<'p>, awaitable: &Bound<'p, PyAny>) -> PyResult<Bound<'p, PyAny>> {
     ENSURE_FUTURE
-        .get_or_try_init(|| -> PyResult<PyObject> {
+        .get_or_try_init(py, || -> PyResult<Py<PyAny>> {
             Ok(asyncio(py)?.getattr("ensure_future")?.into())
         })?
         .bind(py)
         .call1((awaitable,))
 }
 
-fn create_future(event_loop: Bound<PyAny>) -> PyResult<Bound<'_, PyAny>> {
+fn create_future(event_loop: Bound<'_, PyAny>) -> PyResult<Bound<'_, PyAny>> {
     event_loop.call_method0("create_future")
 }
 
@@ -440,9 +436,9 @@ fn close(event_loop: Bound<PyAny>) -> PyResult<()> {
     Ok(())
 }
 
-fn asyncio(py: Python) -> PyResult<&Bound<PyAny>> {
+fn asyncio(py: Python<'_>) -> PyResult<&Bound<'_, PyAny>> {
     ASYNCIO
-        .get_or_try_init(|| Ok(py.import_bound("asyncio")?.into()))
+        .get_or_try_init(py, || Ok(py.import("asyncio")?.into()))
         .map(|asyncio| asyncio.bind(py))
 }
 
@@ -453,7 +449,7 @@ pub fn get_running_loop(py: Python) -> PyResult<Bound<PyAny>> {
     // Ideally should call get_running_loop, but calls get_event_loop for compatibility when
     // get_running_loop is not available.
     GET_RUNNING_LOOP
-        .get_or_try_init(|| -> PyResult<PyObject> {
+        .get_or_try_init(py, || -> PyResult<Py<PyAny>> {
             let asyncio = asyncio(py)?;
 
             Ok(asyncio.getattr("get_running_loop")?.into())
@@ -462,9 +458,9 @@ pub fn get_running_loop(py: Python) -> PyResult<Bound<PyAny>> {
         .call0()
 }
 
-fn contextvars(py: Python) -> PyResult<&Bound<PyAny>> {
+fn contextvars(py: Python<'_>) -> PyResult<&Bound<'_, PyAny>> {
     Ok(CONTEXTVARS
-        .get_or_try_init(|| py.import_bound("contextvars").map(|m| m.into()))?
+        .get_or_try_init(py, || py.import("contextvars").map(|m| m.into()))?
         .bind(py))
 }
 
@@ -476,9 +472,9 @@ fn copy_context(py: Python) -> PyResult<Bound<PyAny>> {
 #[derive(Debug)]
 pub struct TaskLocals {
     /// Track the event loop of the Python task
-    event_loop: PyObject,
+    event_loop: Py<PyAny>,
     /// Track the contextvars of the Python task
-    context: PyObject,
+    context: Py<PyAny>,
 }
 
 impl TaskLocals {
@@ -530,7 +526,7 @@ impl TaskLocals {
 
 #[pyclass]
 struct PyTaskCompleter {
-    tx: Option<oneshot::Sender<PyResult<PyObject>>>,
+    tx: Option<oneshot::Sender<PyResult<Py<PyAny>>>>,
 }
 
 #[pymethods]
@@ -559,14 +555,14 @@ impl PyTaskCompleter {
 
 #[pyclass]
 struct PyEnsureFuture {
-    awaitable: PyObject,
-    tx: Option<oneshot::Sender<PyResult<PyObject>>>,
+    awaitable: Py<PyAny>,
+    tx: Option<oneshot::Sender<PyResult<Py<PyAny>>>>,
 }
 
 #[pymethods]
 impl PyEnsureFuture {
     pub fn __call__(&mut self) -> PyResult<()> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let task = ensure_future(py, self.awaitable.bind(py))?;
             let on_complete = PyTaskCompleter { tx: self.tx.take() };
             task.call_method1("add_done_callback", (on_complete,))?;
@@ -576,14 +572,14 @@ impl PyEnsureFuture {
     }
 }
 
-fn call_soon_threadsafe(
-    event_loop: &Bound<PyAny>,
+fn call_soon_threadsafe<'py>(
+    event_loop: &Bound<'py, PyAny>,
     context: &Bound<PyAny>,
-    args: impl IntoPy<Py<PyTuple>>,
+    args: impl PyCallArgs<'py>,
 ) -> PyResult<()> {
     let py = event_loop.py();
 
-    let kwargs = PyDict::new_bound(py);
+    let kwargs = PyDict::new(py);
     kwargs.set_item("context", context)?;
 
     event_loop.call_method("call_soon_threadsafe", args, Some(&kwargs))?;
@@ -594,8 +590,8 @@ fn call_soon_threadsafe(
 ///
 /// This function converts the `awaitable` into a Python Task using `run_coroutine_threadsafe`. A
 /// completion handler sends the result of this Task through a
-/// `futures::channel::oneshot::Sender<PyResult<PyObject>>` and the future returned by this function
-/// simply awaits the result through the `futures::channel::oneshot::Receiver<PyResult<PyObject>>`.
+/// `futures::channel::oneshot::Sender<PyResult<Py<PyAny>>>` and the future returned by this function
+/// simply awaits the result through the `futures::channel::oneshot::Receiver<PyResult<Py<PyAny>>>`.
 ///
 /// # Arguments
 /// * `locals` - The Python event loop and context to be used for the provided awaitable
@@ -605,6 +601,7 @@ fn call_soon_threadsafe(
 ///
 /// ```
 /// use std::time::Duration;
+/// use std::ffi::CString;
 ///
 /// use pyo3::prelude::*;
 ///
@@ -617,23 +614,23 @@ fn call_soon_threadsafe(
 ///
 /// # #[cfg(feature = "tokio-runtime")]
 /// async fn py_sleep(seconds: f32) -> PyResult<()> {
-///     let test_mod = Python::with_gil(|py| -> PyResult<PyObject> {
+///     let test_mod = Python::attach(|py| -> PyResult<Py<PyAny>> {
 ///         Ok(
-///             PyModule::from_code_bound(
+///             PyModule::from_code(
 ///                 py,
-///                 PYTHON_CODE,
-///                 "test_into_future/test_mod.py",
-///                 "test_mod"
+///                 &CString::new(PYTHON_CODE).unwrap(),
+///                 &CString::new("test_into_future/test_mod.py").unwrap(),
+///                 &CString::new("test_mod").unwrap(),
 ///             )?
 ///             .into()
 ///         )
 ///     })?;
 ///
-///     Python::with_gil(|py| {
+///     Python::attach(|py| {
 ///         pyo3_async_runtimes::into_future_with_locals(
 ///             &pyo3_async_runtimes::tokio::get_current_locals(py)?,
 ///             test_mod
-///                 .call_method1(py, "py_sleep", (seconds.into_py(py),))?
+///                 .call_method1(py, "py_sleep", (seconds,))?
 ///                 .into_bound(py),
 ///         )
 ///     })?
@@ -644,7 +641,7 @@ fn call_soon_threadsafe(
 pub fn into_future_with_locals(
     locals: &TaskLocals,
     awaitable: Bound<PyAny>,
-) -> PyResult<impl Future<Output = PyResult<PyObject>> + Send> {
+) -> PyResult<impl Future<Output = PyResult<Py<PyAny>>> + Send> {
     let py = awaitable.py();
     let (tx, rx) = oneshot::channel();
 
@@ -660,8 +657,8 @@ pub fn into_future_with_locals(
     Ok(async move {
         match rx.await {
             Ok(item) => item,
-            Err(_) => Python::with_gil(|py| {
-                Err(PyErr::from_value_bound(
+            Err(_) => Python::attach(|py| {
+                Err(PyErr::from_value(
                     asyncio(py)?.call_method0("CancelledError")?,
                 ))
             }),
